' ########################################################################################
' Tabbed browser example.
' ########################################################################################

' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "OLECON.INC"     ' // Ole Container
#INCLUDE ONCE "EXDISP.INC"     ' // WebBrowser Control
#INCLUDE ONCE "COMMCTRL.INC"   ' // Common controls
#INCLUDE ONCE "MDI32.INC"      ' // MDI wrappers
#INCLUDE ONCE "SHOBJIDL.INC"   ' // Shell objects
#RESOURCE "TabbedBrowser.pbr"  ' // Resource file

%MAX_TABCAPTION_LEN = 40     ' Maximum length of the tab caption

' ========================================================================================
' Identifiers
' ========================================================================================

%IDC_STATUSBAR   = 101
%IDC_REBAR       = 102
%IDC_IEWB        = 103
%IDC_TOOLBAR     = 104
%IDC_EDITURL     = 105
%IDC_GOBTN       = 106
%IDC_TABMDI      = 107

' Rebar1
%IDS_STRING0     = 32770
%IDS_STRING1     = 32771
%IDS_STRING2     = 32772

' Command
%ID_GOBACK       = 28000
%ID_FORWARD      = 28001
%ID_NEW          = 28002
%ID_FIND         = 28003
%ID_PRINTPREVIEW = 28004
%ID_PRINT        = 28005
%ID_PROPERTIES   = 28006
%ID_FILE_SAVE    = 28007
%ID_PAGESETUP    = 28008
%ID_REFRESH      = 28009
%ID_STOP         = 28010
%ID_ZOOMIN       = 28011
%ID_ZOOMOUT      = 28012
%ID_EXIT         = 28013
%ID_CASCADE      = 28014
%ID_TILEH        = 28015
%ID_TILEV        = 28016
%ID_ARRANGE      = 28017

' Combined equates for Tab control
%TCIF_ALL = %TCIF_TEXT          OR _
            %TCIF_IMAGE         OR _
            %TCIF_RTLREADING    OR _
            %TCIF_PARAM         OR _
            %TCIF_STATE         OR _
            %TCIS_BUTTONPRESSED OR _
            %TCIS_HIGHLIGHTED

' Image list
%IDI_INTERNET32  = 100
%IDI_INTERNET    = 101
%IDI_BACK        = 102
%IDI_FORWARD     = 103
%IDI_NEW         = 104
%IDI_FIND        = 105
%IDI_PRINTPREV   = 106
%IDI_PAGESETUP   = 107
%IDI_PRINT       = 108
%IDI_PROPERTIES  = 109
%IDI_SAVE        = 110
%IDI_REFRESH     = 111
%IDI_STOP        = 112
%IDI_ZOOMIN      = 113
%IDI_ZOOMOUT     = 114
%IDI_EXIT        = 115

' ========================================================================================
' Globals
' ========================================================================================
GLOBAL  ghTabMdi                AS DWORD    ' handle to document tab control
GLOBAL  ghWndClient             AS DWORD    ' MDI main window
GLOBAL  fClosed                 AS LONG     ' Flag

' ========================================================================================
' Creates a new instance of the WebBrowser control
' ========================================================================================
FUNCTION CreateWebBrowser (BYVAL hParent AS DWORD) AS DWORD

   LOCAL hWndChild AS DWORD

   ' Create the WebBrowser control
   hWndChild = CreateWindowEx(%NULL, _                                             ' extended styles
                              $OC_CLASSNAME, _                                     ' class name
                              "Shell.Explorer", _                                  ' caption
                              %WS_CHILD OR %WS_VISIBLE OR %WS_CLIPSIBLINGS OR _    ' window styles
                              %WS_TABSTOP, _
                              0, 0, _                                              ' left, top
                              0, 0, _                                              ' width, height
                              hParent, %IDC_IEWB, _                                ' handle of parent, control ID
                              GetModuleHandle(""), BYVAL %NULL)                    ' handle of instance, creation parameters
   FUNCTION = hWndChild

END FUNCTION
' ========================================================================================

' ========================================================================================
' Retrieves the handle of the MDI active window
' ========================================================================================
FUNCTION GetActiveWbWindow() AS DWORD
  FUNCTION = GetDlgItem(MdiGetActive(ghWndClient), %IDC_IEWB)
END FUNCTION
' ========================================================================================

' ========================================================================================
' Using the Mdi child caption find associated Tab index and activate the associated tab.
' ========================================================================================
FUNCTION EnumMdiTitleToTab (szMdiCaption AS ASCIIZ) AS LONG

   LOCAL nTab      AS LONG
   LOCAL ttc_item  AS TCITEM
   LOCAL idx       AS LONG
   LOCAL szTabTxt  AS ASCIIZ * 256

   nTab = SendMessage(ghTabMdi, %TCM_GETITEMCOUNT, 0, 0)
   FOR idx = 0 TO nTab - 1
      ' Get tab item text string
      ttc_item.mask       = %TCIF_TEXT
      ttc_item.pszText    = VARPTR(szTabTxt)
      ttc_item.cchTextMax = SIZEOF(szTabTxt)
      SendMessage ghTabMdi, %TCM_GETITEM, idx, BYVAL VARPTR(ttc_item)
      IF szMdiCaption = szTabTxt THEN
         SendMessage ghTabMdi, %TCM_SETCURSEL, idx, 0
         FUNCTION = idx
         EXIT FUNCTION
      END IF
    NEXT

END FUNCTION
' ========================================================================================

' ========================================================================================
' Using the Tab caption find the associated Mdi Child handle and making that window the
' active document view.
' ========================================================================================
FUNCTION EnumTabToMdiHandle (BYREF szTabCaption AS ASCIIZ) AS DWORD

   LOCAL hMdi   AS DWORD
   LOCAL szText AS ASCIIZ * %MAX_PATH

   ' Get the first document view handle
   hMdi = GetWindow(ghWndClient, %GW_CHILD)
   ' Cycle thru all the open DocView windows
   WHILE hMdi
      ' Get the document view caption text
      GetWindowText hMdi, szText, %MAX_PATH
      IF szText = szTabCaption THEN
         ' Bring the document view to the front and make it the active window
         IF IsIconic(hMdi) THEN
            SendMessage ghWndClient, %WM_MDIRESTORE, hMdi, 0
         ELSE
            SendMessage ghWndClient, %WM_MDIACTIVATE, hMdi, 0
         END IF
         ' return handle as success
         FUNCTION = hMdi
         EXIT FUNCTION
      END IF
      hMdi = GetWindow(hMdi, %GW_HWNDNEXT)
   WEND

   ' else return false as failure
   FUNCTION = %FALSE

END FUNCTION
' ========================================================================================

' ========================================================================================
' Inserts a new Tab item.
' ========================================================================================
FUNCTION InsertTabMdiItem(BYVAL hTab AS LONG, BYVAL item AS LONG, szTabText AS ASCIIZ) AS LONG

   LOCAL ttc_item AS TCITEM

   IF SendMessage(ghTabMdi, %TCM_GETITEMCOUNT, 0, 0) = 0 THEN
      ShowWindow ghTabMdi, %SW_SHOW
   END IF
   ' Insert a tab...
   ttc_item.mask       = %TCIF_ALL
   ttc_item.pszText    = VARPTR(szTabText)
   ttc_item.cchTextMax = LEN(szTabText)
   ttc_item.iImage     = 0
   ttc_item.lParam     = 0
   FUNCTION = SendMessage(hTab, %TCM_INSERTITEM, item, VARPTR(ttc_item))

END FUNCTION
' ========================================================================================

' ========================================================================================
' Using the Mdi child caption find associated Tab index and delete it
' ========================================================================================
FUNCTION EnumMdiTitleToTabRemove (szMdiCaption AS ASCIIZ) AS LONG

   LOCAL nTab       AS LONG
   LOCAL ttc_item   AS TCITEM
   LOCAL szTabTxt   AS ASCIIZ * 256
   LOCAL idx        AS LONG
   LOCAL itemNext   AS LONG
   LOCAL hMdi       AS DWORD

   nTab = SendMessage(ghTabMdi, %TCM_GETITEMCOUNT, 0, 0)
   IF nTab = 0 THEN FUNCTION = 0 : EXIT FUNCTION
   FOR idx = 0 TO nTab-1
      ' Get tab item text string
      ttc_item.mask       = %TCIF_TEXT
      ttc_item.pszText    = VARPTR(szTabTxt)
      ttc_item.cchTextMax = SIZEOF(szTabTxt)
      SendMessage ghTabMdi, %TCM_GETITEM, idx, VARPTR(ttc_item)
      IF szMdiCaption = szTabTxt THEN
         ' Delete this tab
         SendMessage ghTabMdi, %TCM_DELETEITEM, idx, 0
         ' Find next available MDI docview in hiarchy and activate it...
         hMdi = SendMessage(ghWndClient, %WM_MDIGETACTIVE, 0, 0)
         IF hMdi THEN
            ' Using handle, get associated tab via caption string...
            LOCAL szText AS ASCIIZ*%MAX_PATH
            ' Get the DocView caption text
            GetWindowText hMdi, szText, %MAX_PATH
            ' Find the associated tab and activate it
            EnumMdiTitleToTab szText
         END IF
         ' Get and activate associated Document view
         itemNext = SendMessage(ghTabMdi,%TCM_GETCURSEL,0,0)
         ttc_item.mask       = %TCIF_TEXT
         ttc_item.pszText    = VARPTR(szTabTxt)
         ttc_item.cchTextMax = SIZEOF(szTabTxt)
         SendMessage ghTabMdi, %TCM_GETITEM, itemNext, VARPTR(ttc_item)
         EnumTabToMdiHandle szTabTxt
         IF SendMessage(ghTabMdi, %TCM_GETITEMCOUNT, 0, 0) = 0 THEN
            ShowWindow ghTabMdi, %SW_HIDE
         END IF
         ' Return item next...
         FUNCTION = itemNext
         EXIT FUNCTION
      END IF
   NEXT

END FUNCTION
' ========================================================================================

' ========================================================================================
' Changes the name of the tab
' ========================================================================================
SUB SetTabName (BYVAL nTab AS LONG, BYVAL strName AS STRING)

   LOCAL ttc_item AS TCITEM

   ttc_item.mask       = %TCIF_TEXT
   ttc_item.pszText    = STRPTR(strName)
   ttc_item.cchTextMax = 256
   SendMessage ghTabMdi, %TCM_SETITEM, nTab, BYVAL VARPTR(ttc_item)

END SUB
' ========================================================================================

' ========================================================================================
' Creates a Tab control for the Mdi GUI interface.
' ========================================================================================
FUNCTION CreateTabMdiCtl (BYVAL hWnd AS LONG, BYVAL hFont AS DWORD) AS DWORD

   LOCAL rc AS RECT
   LOCAL hTabImageList AS DWORD
   LOCAL hIcon AS DWORD

   GetWindowRect hWnd, rc
   ghTabMdi = CreateWindowEx(0, "SysTabControl32","",_
                             %WS_CHILD OR %TCS_FOCUSNEVER OR %WS_CLIPCHILDREN _
                             OR %TCS_BOTTOM OR %TCS_TABS OR %TCS_SINGLELINE, _
                             rc.nLeft, _
                             rc.nTop, _
                             rc.nRight, _
                             22, _
                             hWnd, %IDC_TABMDI, GetModuleHandle(""), BYVAL %NULL)
   IF hFont THEN SendMessage ghTabMdi, %WM_SETFONT, hFont, %TRUE
   hTabImageList = ImageList_Create(16, 16, %ILC_MASK, 2, 1)
   hIcon = LoadIcon(GetModuleHandle(""), BYVAL 101)
   ImageList_AddIcon hTabImageList, hIcon
   DestroyIcon hIcon
   SendMessage(ghTabMdi, %TCM_SETIMAGELIST, 0, hTabImageList)
   FUNCTION = ghTabMdi

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WinMain ( _
   BYVAL hInstance     AS DWORD, _       ' handle of current instance
   BYVAL hPrevInstance AS DWORD, _       ' handle of previous instance(not used in Win32)
   BYVAL pszCmdLine    AS ASCIIZ PTR, _  ' address of command line
   BYVAL nCmdShow      AS LONG _         ' show state of window
   ) AS LONG

   LOCAL szClassName AS ASCIIZ * %MAX_PATH       ' class name
   LOCAL wcex        AS WNDCLASSEX               ' class information
   LOCAL ticc        AS INIT_COMMON_CONTROLSEX   ' specifies common control classes to register
   LOCAL hWndMain    AS DWORD
   LOCAL cxIcon      AS LONG                     ' width of a small icon in pixels
   LOCAL cyIcon      AS LONG                     ' height of a small icon in pixels

   ' Initilize the COM library using OleInitialize to allow cut and paste
   OleInitialize 0

   ' Required: Initialize the Ole Container
   OC_WinInit

   ' Register the main window
   szClassName        = "TabbedBrowser"
   wcex.cbSize        = SIZEOF(wcex)                                ' size of WNDCLASSEX structure
   wcex.style         = %CS_DBLCLKS                                 ' class styles
   wcex.lpfnWndProc   = CODEPTR(WndProc)                            ' address of window procedure used by class
   wcex.cbClsExtra    = 0                                           ' extra class bytes
   wcex.cbWndExtra    = 0                                           ' extra window bytes
   wcex.hInstance     = hInstance                                   ' instance of the process that is registering the window
   wcex.hIcon         = LoadIcon(hInstance, BYVAL %IDI_INTERNET32) ' handle of class icon
   wcex.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)         ' handle of class cursor
   wcex.hbrBackground = %COLOR_BTNFACE + 1                          ' brush used to fill background of window's client area
   wcex.lpszMenuName  = %NULL                                       ' resource identifier of the class menu
   wcex.lpszClassName = VARPTR(szClassName)                         ' class name
   wcex.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_INTERNET)    ' handle of small icon shown in caption/system Taskbar

   IF ISFALSE RegisterClassEx(wcex) THEN EXIT FUNCTION

   ' Register the windows clas for MDI windows
   szClassName       = "WB"
   wcex.cbSize        = SIZEOF(wcex)
   wcex.style         = %CS_DBLCLKS
   wcex.lpfnWndProc   = CODEPTR(MdiWindowProc)
   wcex.cbClsExtra    = 0
   wcex.cbWndExtra    = 0
   wcex.hInstance     = hInstance
   wcex.hIcon         = LoadIcon(%NULL, BYVAL %IDI_INTERNET32)
   wcex.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
   wcex.hbrBackground = %COLOR_WINDOW + 1
   wcex.lpszMenuName  = %NULL
   wcex.lpszClassName = VARPTR(szClassName)
   wcex.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_INTERNET)

   IF ISFALSE RegisterClassEx(wcex) THEN EXIT FUNCTION

   ' Load the common controls library and specify the classes to register.
   ticc.dwSize = SIZEOF(ticc)
   ticc.dwICC  = %ICC_BAR_CLASSES OR %ICC_COOL_CLASSES
   InitCommonControlsEx ticc

   ' Create the Form1 window
   hWndMain = CreateWindowEx(%WS_EX_WINDOWEDGE, _                                          ' extended styles
                             "TabbedBrowser", _                                              ' class name
                             "Tabbed WebBrowser", _                                       ' caption
                             %WS_OVERLAPPEDWINDOW OR %WS_VISIBLE, _                        ' window styles
                             0, 0, _                                                       ' left, top
                             800, 568, _                                                       ' width, height
                             %NULL, %NULL, _                                               ' handle of owner, menu handle
                             hInstance, BYVAL %NULL)                                      ' handle of instance, creation parameters
   IF ISFALSE hWndMain THEN EXIT FUNCTION

   ' Make the window visible; update its client area
   ShowWindow hWndMain, nCmdShow
   UpdateWindow hWndMain

   ' Message handler loop
   LOCAL uMsg AS tagMsg
   LOCAL hWndModeless AS DWORD
   WHILE GetMessage(uMsg, %NULL, 0, 0)
      IF ISFALSE OC_ForwardMessage(GetFocus, uMsg) THEN
         hWndModeless = phnxGetFormHandle(GetFocus)
         IF (ISFALSE hWndModeless) OR (ISFALSE IsDialogMessage(hWndModeless, uMsg)) THEN
            TranslateMessage uMsg
            DispatchMessage uMsg
         END IF
      END IF
   WEND

   ' Uninitialize the COM library
   OleUninitialize

   FUNCTION = uMsg.wParam

END FUNCTION
' ========================================================================================

' ========================================================================================
' FUNCTION: phnxGetFormHandle
' PURPOSE:  Finds the handle of the top-level window or MDI child
'           window that is the ancestor of the specified window.  The
'           reference handle is the handle of any control on the form.
' RETURNS:  The handle of the form.
' ========================================================================================

FUNCTION phnxGetFormHandle ( _
   BYVAL hWnd  AS DWORD _  ' reference handle
   ) AS DWORD

   WHILE ISTRUE (GetWindowLong(hWnd, %GWL_STYLE) AND %WS_CHILD)
      IF ISTRUE (GetWindowLong(hWnd, %GWL_EXSTYLE) AND %WS_EX_MDICHILD) THEN EXIT LOOP
      hWnd = GetParent(hWnd)
   WEND

   FUNCTION = hWnd

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Window procedure
' ========================================================================================
FUNCTION WndProc ( _
   BYVAL hWnd    AS DWORD, _ ' window handle
   BYVAL uMsg    AS DWORD, _ ' type of message
   BYVAL wParam  AS DWORD, _ ' first message parameter
   BYVAL lParam  AS LONG _   ' second message parameter
   ) AS LONG

   LOCAL szItem        AS ASCIIZ * %MAX_PATH   ' working variable
   LOCAL trbi          AS REBARINFO            ' specifies attributes(imagelist) of the rebar control
   LOCAL trbbi         AS REBARBANDINFO        ' specifies or receives the attributes of a rebar band
   LOCAL ttbb          AS TBBUTTON             ' specifies or receives the attributes of a toolbar button
   LOCAL ttbab         AS TBADDBITMAP          ' specifies the images to add to a toolbar
   LOCAL ptnmhdr       AS NMHDR PTR            ' information about a notification message
   LOCAL ptttdi        AS NMTTDISPINFO PTR     ' tooltip notification message information
   LOCAL pttbb         AS TBBUTTON PTR         ' address of array of toolbar button info
   LOCAL plEdge        AS LONG PTR             ' address of array of right edges
   LOCAL hCtrl         AS DWORD                ' handle of child window
   LOCAL hWndRebar     AS DWORD                ' handle of rebar control
   LOCAL hFont         AS DWORD                ' handle of font used by form
   LOCAL hImage        AS DWORD                ' handle of the image
   LOCAL hImageList    AS DWORD                ' handle of the toolbar image list

   ' MDI
   LOCAL lpNmh         AS NMHDR PTR
   LOCAL ttc_item      AS TC_ITEM
   LOCAL hMdi          AS DWORD
   LOCAL sel           AS LONG
   LOCAL szTabTxt      AS ASCIIZ * 2555
   LOCAL cc            AS CLIENTCREATESTRUCT

   ' Menu
   LOCAL hMenu         AS DWORD
   LOCAL hSubmenu      AS DWORD
   LOCAL hWndFirst     AS DWORD
   LOCAL hWndPrev      AS DWORD
   LOCAL hWndActive    AS DWORD
   STATIC hMenuWindow  AS DWORD

   LOCAL pIWebBrowser2 AS IWebBrowser2
   LOCAL vUrl          AS VARIANT
   LOCAL szUrl         AS ASCIIZ * 2048        ' %INTERNET_MAX_PATH_LENGTH
   LOCAL nScale        AS LONG
   LOCAL nRange        AS LONG
   LOCAL vScale        AS VARIANT
   LOCAL vRange        AS VARIANT

   SELECT CASE uMsg

'      CASE %WM_NCACTIVATE
'         STATIC hWndSaveFocus AS DWORD
'         IF wParam = 0 THEN
'            ' Save control focus
'            hWndSaveFocus = GetFocus()
'         ELSEIF hWndSaveFocus THEN
'            ' Restore control focus
'            SetFocus(hWndSaveFocus)
'            hWndSaveFocus = 0
'         END IF

      CASE %WM_ACTIVATE
         STATIC hWndSaveFocus AS DWORD
         IF LO(WORD, wParam) = %WA_INACTIVE THEN
            ' Save the control with the keyboard focus
            hWndSaveFocus = GetFocus()
         ELSEIF hWndSaveFocus THEN
            ' Set the keyboard focus to the control with
            ' the focus when the window was deactivated
            SetFocus(hWndSaveFocus)
            hWndSaveFocus = 0
         END IF
         EXIT FUNCTION


      CASE %WM_SYSCOMMAND
         ' Capture this message and send a WM_CLOSE message
         IF (wParam AND &HFFF0) = %SC_CLOSE THEN
            SendMessage hWnd, %WM_CLOSE, 0, 0
            EXIT FUNCTION
         END IF

      CASE %WM_COMMAND

         SELECT CASE LO(WORD, wParam)

            CASE %IDOK
               ' If we are in the URL edit control, load the web page
               hCtrl = GetDlgItem(GetDlgItem(hWnd, %IDC_REBAR), %IDC_EDITURL)
               IF GetFocus = hCtrl THEN
                  PostMessage hWnd, %WM_COMMAND, %IDC_GOBTN, MAK(DWORD, %BN_CLICKED, %IDC_GOBTN)
                  EXIT FUNCTION
               END IF

            CASE %ID_EXIT
               SendMessage hWnd, %WM_CLOSE, 0, 0
               EXIT FUNCTION

            CASE %ID_CASCADE
               MdiCascade ghWndClient
               EXIT FUNCTION

            CASE %ID_TILEH
               SendMessage(ghWndClient, %WM_MDITILE, %MDITILE_HORIZONTAL, 0)
               EXIT FUNCTION

            CASE %ID_TILEV
               SendMessage(ghWndClient, %WM_MDITILE, %MDITILE_VERTICAL, 0)
               EXIT FUNCTION

            CASE %ID_ARRANGE
               MdiIconArrange ghWndClient
               EXIT FUNCTION

            CASE %ID_NEW
               ' Create an MDI child
               hMdi = CreateMdiChild("WB", ghWndClient, "", %WS_MAXIMIZE)
               ' Get the handle of the window that hosts the webbrowser
               hCtrl = GetDlgItem(hMdi, %IDC_IEWB)
               vUrl = "about:blank"
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.Navigate2 vUrl
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %IDC_GOBTN
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  ' Get the Url
                  hCtrl = GetDlgItem(GetDlgItem(hWnd, %IDC_REBAR), %IDC_EDITURL)
                  GetWindowText hCtrl, szUrl, SIZEOF(szUrl)
                  vUrl = szUrl
                  IF ISFALSE GetActiveWbWindow THEN
                     ' Create an MDI child
                     hMdi = CreateMdiChild("WB", ghWndClient, "", %WS_MAXIMIZE)
                     ' Get the handle of the window that hosts the webbrowser
                     hCtrl = GetDlgItem(hMdi, %IDC_IEWB)
                  ELSE
                     ' Use the active window
                     hCtrl = GetActiveWbWindow
                  END IF
                  pIWebBrowser2 = OC_GetDispatch(hCtrl)
                  IF ISOBJECT(pIWebBrowser2) THEN
                     pIWebBrowser2.Navigate2 vUrl
                     pIWebBrowser2 = NOTHING
                  END IF
                  EXIT FUNCTION
               END IF

            CASE %ID_GOBACK
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.GoBack
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_FORWARD
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.GoForward
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_FIND
              ' Warning: This code uses an undocumented command-group GUID that is
              ' subject to change in the future. Currently it works in all versions of
              ' Internet Explorer up to 7. See http://support.microsoft.com/?kbid=311288
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  LOCAL CGID_WebBrowser AS GUID
                  LOCAL pDisp AS IDispatch
                  LOCAL pCmdTarget AS IOleCommandTarget
                  LOCAL vIn AS VARIANT
                  LOCAL vOut AS VARIANT
                  CGID_WebBrowser = GUID$("{ED016940-BD5B-11CF-BA4E-00C04FD70816}")
                  pDisp = pIWebBrowser2.Document
                  IF ISOBJECT(pDisp) THEN
                     pCmdTarget = pDisp
                     IF ISOBJECT(pCmdTarget) THEN
                        pCmdTarget.Exec(CGID_WebBrowser, 1, 0, vIn, vOut)
                     END IF
                  END IF
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_PRINTPREVIEW
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_PRINTPREVIEW, %OLECMDEXECOPT_PROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_PAGESETUP
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_PAGESETUP, %OLECMDEXECOPT_PROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_PRINT
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_PRINT, %OLECMDEXECOPT_PROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_PROPERTIES
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_PROPERTIES, %OLECMDEXECOPT_PROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_FILE_SAVE
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_SAVEAS, %OLECMDEXECOPT_PROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_REFRESH
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_REFRESH, %OLECMDEXECOPT_DONTPROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_STOP
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  pIWebBrowser2.ExecWB %OLECMDID_STOP, %OLECMDEXECOPT_DONTPROMPTUSER
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_ZOOMIN
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  ' Call the ExecWB method to get the current font zoom scale
                  pIWebBrowser2.ExecWB %OLECMDID_ZOOM, %OLECMDEXECOPT_DONTPROMPTUSER, BYVAL %NULL, vScale
                  nScale = VARIANT#(vScale)
                  ' Call the ExecWB method to get the upper limit of zoom range (0-4).
                  pIWebBrowser2.ExecWB %OLECMDID_GETZOOMRANGE, %OLECMDEXECOPT_DONTPROMPTUSER, BYVAL %NULL, vRange
                  nRange = VARIANT#(vRange)
                  IF nScale < nRange THEN nScale +=1 ELSE nScale = nRange
                  ' Call the ExecWB method to set the new font zoom scale
                  vScale = nScale AS LONG   ' The 'AS LONG' part is important!
                  pIWebBrowser2.ExecWB %OLECMDID_ZOOM, %OLECMDEXECOPT_DONTPROMPTUSER, vScale
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %ID_ZOOMOUT
               hCtrl = GetActiveWbWindow
               pIWebBrowser2 = OC_GetDispatch(hCtrl)
               IF ISOBJECT(pIWebBrowser2) THEN
                  ' Call the ExecWB method to get the current font zoom scale
                  pIWebBrowser2.ExecWB %OLECMDID_ZOOM, %OLECMDEXECOPT_DONTPROMPTUSER, BYVAL %NULL, vScale
                  nScale = VARIANT#(vScale)
                  IF nScale > 0 THEN nScale -=1 ELSE nScale = 0
                  ' Call the ExecWB method to set the new font zoom scale
                  vScale = nScale AS LONG   ' The 'AS LONG' part is important!
                  pIWebBrowser2.ExecWB %OLECMDID_ZOOM, %OLECMDEXECOPT_DONTPROMPTUSER, vScale
                  pIWebBrowser2 = NOTHING
               END IF
               EXIT FUNCTION

            CASE %IDC_EDITURL
               SELECT CASE HI(WORD, wParam)
                  CASE %EN_SETFOCUS
                     ' Select all the text of the edit box
                     PostMessage lParam, %EM_SETSEL, 0, -1
                     EXIT FUNCTION
               END SELECT

         END SELECT

      CASE %WM_NOTIFY
         ptnmhdr = lParam

         SELECT CASE @ptnmhdr.code
            CASE %TTN_GETDISPINFO
               ptttdi        = lParam
               @ptttdi.hinst = %NULL

               SELECT CASE @ptttdi.hdr.hwndFrom
                  hCtrl = GetDlgItem(GetDlgItem(hWnd, %IDC_REBAR), %IDC_TOOLBAR)
                  CASE SendMessage(hCtrl, %TB_GETTOOLTIPS, 0, 0)
                     SELECT CASE @ptttdi.hdr.idFrom
                        CASE %ID_GOBACK
                           szItem = "Go Back"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_FORWARD
                           szItem = "Go Forward"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_NEW
                           szItem = "New tab"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_FIND
                           szItem = "Find"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_PRINTPREVIEW
                           szItem = "Print Preview"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_PAGESETUP
                           szItem = "Page Setup"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_PRINT
                           szItem = "Print"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_PROPERTIES
                           szItem = "Properties"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_FILE_SAVE
                           szItem = "Save As"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_REFRESH
                           szItem = "Refresh"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_STOP
                           szItem = "Stop"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_ZOOMIN
                           szItem = "Zoom in"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_ZOOMOUT
                           szItem = "Zoom out"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                        CASE %ID_EXIT
                           szItem = "Exit"
                           @ptttdi.lpszText = VARPTR(szItem)
                           EXIT FUNCTION
                     END SELECT
               END SELECT

         END SELECT

         ' MDI
         lpNmh = lParam
         SELECT CASE @lpNmh.Code ' Examine .Code member
            CASE %TCN_LAST TO %TCN_FIRST ' Tab control notifications
               SELECT CASE @lpNmh.idFrom
                  CASE %IDC_TABMDI
                     SELECT CASE @lpNmh.Code
                        CASE %TCN_SELCHANGE ' identify which tab
                           sel = SendMessage(ghTabMdi, %TCM_GETCURSEL, 0, 0)
                           ttc_item.mask       = %TCIF_TEXT
                           ttc_item.pszText    = VARPTR(szTabTxt)
                           ttc_item.cchTextMax = SIZEOF(szTabTxt)
                           SendMessage ghTabMdi, %TCM_GETITEM, sel, BYVAL VARPTR(ttc_item)
                           ' get and activate associated Document view
                           hMdi = EnumTabToMdiHandle(szTabTxt)
                     END SELECT
               END SELECT
         END SELECT

      CASE %WM_SYSCOLORCHANGE
         ' Forward this message to common controls so that they will
         ' be properly updated when the user changes the color settings.
         SendMessage GetDlgItem(hWnd, %IDC_STATUSBAR), %WM_SYSCOLORCHANGE, wParam, lParam
         hWndRebar = GetDlgItem(hWnd, %IDC_REBAR)
         SendMessage hWndRebar, %WM_SYSCOLORCHANGE, wParam, lParam
         SendMessage GetDlgItem(hWndRebar, %IDC_TOOLBAR), %WM_SYSCOLORCHANGE, wParam, lParam

      CASE %WM_SETFOCUS
         ' Set the keyboard focus to the first control that is
         ' visible, not disabled, and has the WS_TABSTOP style
         SetFocus GetNextDlgTabItem(hWnd, %NULL, %FALSE)
         EXIT FUNCTION

      CASE %WM_CLOSE
         hWndFirst = MdiGetActive(ghWndClient)
         hWndActive = hWndFirst
         hWndPrev = 0
         DO WHILE hWndActive
            IF SendMessage(MdiGetActive(ghWndClient), %WM_CLOSE, 0, 0) THEN
               EXIT DO
            END IF
            IF fClosed THEN
               IF hWndFirst = hWndActive THEN
                  IF hWndFirst = hWndPrev THEN
                     hWndFirst = 0
                     hWndPrev = 0
                  ELSE
                     hWndFirst = hWndPrev
                  END IF
               END IF
            ELSE
               IF hWndPrev = 0 THEN
                  hWndPrev = hWndActive
               END IF
               IF hWndFirst = 0 THEN
                  hWndFirst = hWndActive
               END IF
            END IF
            MdiNext ghWndClient, hWndActive, 0&
            hWndActive = MdiGetActive(ghWndClient)
         LOOP UNTIL hWndActive = hWndFirst
         IF hWndActive THEN EXIT FUNCTION

      CASE %WM_DESTROY
         ' Destroy the image list used by the toolbar
         hCtrl = GetDlgItem(GetDlgItem(hWnd, %IDC_REBAR), %IDC_TOOLBAR)
         ImageList_Destroy(SendMessage(hCtrl, %TB_SETIMAGELIST, 0, %NULL))
         ' Destroy the image list used by the tab control
         ImageList_Destroy(SendMessage(ghTabMdi, %TCM_SETIMAGELIST, 0, %NULL))
         ' Quit the application
         PostQuitMessage 0
         EXIT FUNCTION

      CASE %WM_SIZE
         IF wParam <> %SIZE_MINIMIZED THEN
            LOCAL rc AS RECT
            LOCAL rbh AS LONG
            LOCAL sbh AS LONG
            LOCAL tch AS LONG
            SendMessage GetDlgItem(hWnd, %IDC_STATUSBAR), %WM_SIZE, wParam, lParam
            InvalidateRect GetDlgItem(hWnd, %IDC_STATUSBAR), BYVAL %NULL, %TRUE
            SendMessage GetDlgItem(hWnd, %IDC_REBAR), %WM_SIZE, wParam, lParam
            ' Height of the rebar
            GetClientRect GetDlgItem(hWnd, %IDC_REBAR), rc
            rbh = rc.nBottom - rc.nTop
            ' Height of the statusbar
            GetClientRect GetDlgItem(hWnd, %IDC_STATUSBAR), rc
            sbh = rc.nBottom - rc.nTop
            ' Height of the tab control
            GetClientRect ghTabMdi, rc
            tch = rc.nBottom - rc.nTop
            ' Move the tab control
            MoveWindow ghTabMdi, 0, rbh + 1, LO(WORD, lParam), tch, %TRUE
            ' Move the MDI main window
            MoveWindow ghWndClient, 0, rbh + tch + 1, LO(WORD, lParam), HI(WORD, lParam) - rbh - sbh - tch, %TRUE
            EXIT FUNCTION
         END IF

      CASE %WM_CREATE
         ' Create font used by container
         hFont = GetStockObject(%DEFAULT_GUI_FONT)

         ' Create the menu
         hMenu = CreateMenu
         hSubMenu = CreatePopUpMenu
         AppendMenu hMenu, %MF_POPUP OR %MF_ENABLED, hSubMenu, "&File"
         AppendMenu hSubMenu, %MF_ENABLED, %ID_NEW, "&New Tab"
         AppendMenu hSubMenu, %MF_SEPARATOR, 0, ""
         AppendMenu hSubMenu, %MF_ENABLED, %ID_EXIT, "E&xit" + $TAB + "Alt+F4"
         SetMenu hWnd, hMenu
         hMenuWindow = CreatePopUpMenu
         AppendMenu hMenu, %MF_POPUP OR %MF_ENABLED, hMenuWindow, "&Window"
         AppendMenu hMenuWindow, %MF_ENABLED, %ID_CASCADE, "&Cascade"
         AppendMenu hMenuWindow, %MF_ENABLED, %ID_TILEH, "Tile &Horizontal"
         AppendMenu hMenuWindow, %MF_ENABLED, %ID_TILEV, "Tile &Vertical"
         AppendMenu hMenuWindow, %MF_ENABLED, %ID_ARRANGE, "&Arrange icons"
         SetMenu hWnd, hMenu

         ' Create the Statusbar statusbar control
         hCtrl = CreateWindowEx(%NULL, _                                             ' extended styles
                                    "msctls_statusbar32", _                              ' class name
                                    "", _                                                ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR _                        ' window styles
                                    %CCS_BOTTOM OR %SBARS_SIZEGRIP, _                    ' class styles
                                    0, 347, _                                            ' left, top
                                    535, 23, _                                           ' width, height
                                    hWnd, %IDC_STATUSBAR, _                       ' handle of parent, control ID
                                    GetModuleHandle(""), BYVAL %NULL)                    ' handle of instance, creation parameters
         SendMessage hCtrl, %WM_SETFONT, hFont, %TRUE

         ' Allocate memory for the coordinate of the right edge of each part
         plEdge = HeapAlloc(GetProcessHeap(), %HEAP_ZERO_MEMORY, 1 * 4)
         IF plEdge THEN
            @plEdge[0] = -1
            SendMessage hCtrl, %SB_SETPARTS, 1, BYVAL plEdge
            ' Free memory that was allocated for the edge info
            HeapFree GetProcessHeap(), 0, BYVAL plEdge
         END IF
         ' Update the size of the statusbar
         SendMessage hCtrl, %WM_SIZE, 0, 0

         ' Create the Rebar rebar control
         hCtrl = CreateWindowEx(%NULL, _                                             ' extended styles
                                    "ReBarWindow32", _                                   ' class name
                                    "", _                                                ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_BORDER OR _          ' window styles
                                    %WS_CLIPCHILDREN OR %WS_CLIPSIBLINGS OR _
                                    %CCS_NOPARENTALIGN OR %CCS_NODIVIDER OR _            ' class styles
                                    %RBS_VARHEIGHT OR %RBS_BANDBORDERS, _
                                    0, 0, _                                              ' left, top
                                    790, 30, _                                           ' width, height
                                    hWnd, %IDC_REBAR, _                           ' handle of parent, control ID
                                    GetModuleHandle(""), BYVAL %NULL)                    ' handle of instance, creation parameters
         ' Save the handle of the rebar.  It is used when embedding controls
         hWndRebar = hCtrl
         SendMessage hCtrl, %WM_SETFONT, hFont, %TRUE

         ' Create the Toolbar toolbar control
         hCtrl = CreateWindowEx(%NULL, _                                             ' extended styles
                                    "ToolbarWindow32", _                                 ' class name
                                    "", _                                                ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR _                        ' window styles
                                    %CCS_NORESIZE OR %CCS_NODIVIDER OR _                 ' class styles
                                    %TBSTYLE_TOOLTIPS OR %TBSTYLE_FLAT, _
                                    0, 4, _                                              ' left, top
                                    204, 21, _                                           ' width, height
                                    hWnd, %IDC_TOOLBAR, _                         ' handle of parent, control ID
                                    GetModuleHandle(""), BYVAL %NULL)                    ' handle of instance, creation parameters
         SendMessage hCtrl, %WM_SETFONT, hFont, %TRUE

         ' Create and initialize the image list
         hImageList = ImageList_Create(16, 16, %ILC_MASK OR %ILC_COLOR24, 14, 0)
         IF hImageList THEN
            ' Set the background color to use for drawing images
            ImageList_SetBkColor hImageList, %CLR_NONE
            ' Add the images to the imagelist
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_BACK, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_FORWARD, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_NEW, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_FIND, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_PRINTPREV, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_PAGESETUP, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_PRINT, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_PROPERTIES, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_SAVE, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_REFRESH, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_STOP, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_ZOOMIN, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_ZOOMOUT, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
            hImage = LoadImage(GetModuleHandle(""), BYVAL %IDI_EXIT, %IMAGE_ICON, 16, 16, %LR_DEFAULTCOLOR)
            ImageList_ReplaceIcon hImageList, -1, hImage
            DeleteObject hImage
         END IF

         ' Set the imagelist used with default images
         SendMessage hCtrl, %TB_SETIMAGELIST, 0, hImageList

         ' Allocate memory for the button info array
         pttbb = HeapAlloc(GetProcessHeap(), %HEAP_ZERO_MEMORY, 14 * SIZEOF(ttbb))
         IF pttbb THEN
           ' Send the TB_BUTTONSTRUCTSIZE message, for backward compatibility
           SendMessage hCtrl, %TB_BUTTONSTRUCTSIZE, SIZEOF(ttbb), 0
            ' Set the size of the bitmaps
            SendMessage hCtrl, %TB_SETBITMAPSIZE, 0, MAKLNG(16, 16)
           ' Add buttons to the toolbar
           @pttbb[0].iBitmap   = 0
           @pttbb[0].idCommand = %ID_GOBACK
           @pttbb[0].fsState   = %TBSTATE_ENABLED
           @pttbb[0].fsStyle   = %BTNS_BUTTON
           @pttbb[0].dwData    = 0
           @pttbb[0].iString   = -1

           @pttbb[1].iBitmap   = 1
           @pttbb[1].idCommand = %ID_FORWARD
           @pttbb[1].fsState   = %TBSTATE_ENABLED
           @pttbb[1].fsStyle   = %BTNS_BUTTON
           @pttbb[1].dwData    = 0
           @pttbb[1].iString   = -1

           @pttbb[2].iBitmap   = 2
           @pttbb[2].idCommand = %ID_NEW
           @pttbb[2].fsState   = %TBSTATE_ENABLED
           @pttbb[2].fsStyle   = %BTNS_BUTTON
           @pttbb[2].dwData    = 0
           @pttbb[2].iString   = -1

           @pttbb[3].iBitmap   = 3
           @pttbb[3].idCommand = %ID_FIND
           @pttbb[3].fsState   = %TBSTATE_ENABLED
           @pttbb[3].fsStyle   = %BTNS_BUTTON
           @pttbb[3].dwData    = 0
           @pttbb[3].iString   = -1

           @pttbb[4].iBitmap   = 4
           @pttbb[4].idCommand = %ID_PRINTPREVIEW
           @pttbb[4].fsState   = %TBSTATE_ENABLED
           @pttbb[4].fsStyle   = %BTNS_BUTTON
           @pttbb[4].dwData    = 0
           @pttbb[4].iString   = -1

           @pttbb[5].iBitmap   = 5
           @pttbb[5].idCommand = %ID_PAGESETUP
           @pttbb[5].fsState   = %TBSTATE_ENABLED
           @pttbb[5].fsStyle   = %BTNS_BUTTON
           @pttbb[5].dwData    = 0
           @pttbb[5].iString   = -1

           @pttbb[6].iBitmap   = 6
           @pttbb[6].idCommand = %ID_PRINT
           @pttbb[6].fsState   = %TBSTATE_ENABLED
           @pttbb[6].fsStyle   = %BTNS_BUTTON
           @pttbb[6].dwData    = 0
           @pttbb[6].iString   = -1

           @pttbb[7].iBitmap   = 7
           @pttbb[7].idCommand = %ID_PROPERTIES
           @pttbb[7].fsState   = %TBSTATE_ENABLED
           @pttbb[7].fsStyle   = %BTNS_BUTTON
           @pttbb[7].dwData    = 0
           @pttbb[7].iString   = -1

           @pttbb[8].iBitmap   = 8
           @pttbb[8].idCommand = %ID_FILE_SAVE
           @pttbb[8].fsState   = %TBSTATE_ENABLED
           @pttbb[8].fsStyle   = %BTNS_BUTTON
           @pttbb[8].dwData    = 0
           @pttbb[8].iString   = -1

           @pttbb[9].iBitmap   = 9
           @pttbb[9].idCommand = %ID_REFRESH
           @pttbb[9].fsState   = %TBSTATE_ENABLED
           @pttbb[9].fsStyle   = %BTNS_BUTTON
           @pttbb[9].dwData    = 0
           @pttbb[9].iString   = -1

           @pttbb[10].iBitmap   = 10
           @pttbb[10].idCommand = %ID_STOP
           @pttbb[10].fsState   = %TBSTATE_ENABLED
           @pttbb[10].fsStyle   = %BTNS_BUTTON
           @pttbb[10].dwData    = 0
           @pttbb[10].iString   = -1

           @pttbb[11].iBitmap   = 11
           @pttbb[11].idCommand = %ID_ZOOMIN
           @pttbb[11].fsState   = %TBSTATE_ENABLED
           @pttbb[11].fsStyle   = %BTNS_BUTTON
           @pttbb[11].dwData    = 0
           @pttbb[11].iString   = -1

           @pttbb[12].iBitmap   = 12
           @pttbb[12].idCommand = %ID_ZOOMOUT
           @pttbb[12].fsState   = %TBSTATE_ENABLED
           @pttbb[12].fsStyle   = %BTNS_BUTTON
           @pttbb[12].dwData    = 0
           @pttbb[12].iString   = -1

           @pttbb[13].iBitmap   = 13
           @pttbb[13].idCommand = %ID_EXIT
           @pttbb[13].fsState   = %TBSTATE_ENABLED
           @pttbb[13].fsStyle   = %BTNS_BUTTON
           @pttbb[13].dwData    = 0
           @pttbb[13].iString   = -1

           SendMessage hCtrl, %TB_ADDBUTTONS, 14, BYVAL pttbb
           ' Free memory that was allocated for the button info
           HeapFree GetProcessHeap(), 0, BYVAL pttbb
           ' Update the size of the toolbar
           SendMessage hCtrl, %TB_AUTOSIZE, 0, 0
         END IF

         ' Add the band containing the Toolbar1 toolbar control to the rebar
         trbbi.cbSize     = SIZEOF(trbbi)
         trbbi.fMask      = %RBBIM_STYLE OR %RBBIM_CHILD OR %RBBIM_CHILDSIZE OR _
                            %RBBIM_SIZE OR %RBBIM_ID OR %RBBIM_IDEALSIZE
         trbbi.fStyle     = %RBBS_FIXEDSIZE OR %RBBS_CHILDEDGE
         trbbi.hWndChild  = hCtrl
         trbbi.cxMinChild = 110
         trbbi.cyMinChild = 21
         trbbi.cx         = 110
         trbbi.wID        = %IDS_STRING0
         trbbi.cxIdeal    = 110
         SendMessage hWndRebar, %RB_INSERTBAND, -1, BYVAL VARPTR(trbbi)

         ' Create the EditUrl edit control
         hCtrl = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                 ' extended styles
                                    "Edit", _                                            ' class name
                                    "", _                                                ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _         ' window styles
                                    %ES_LEFT OR %ES_AUTOHSCROLL, _                       ' class styles
                                    151, 4, _                                            ' left, top
                                    350, 21, _                                           ' width, height
                                    hWnd, %IDC_EDITURL, _                          ' handle of parent, control ID
                                    GetModuleHandle(""), BYVAL %NULL)                    ' handle of instance, creation parameters
         SendMessage hCtrl, %WM_SETFONT, hFont, %TRUE
         SetWindowText hCtrl, "http://www.jose.it-berater.org/index.html"

         ' Add the band containing the EditUrl edit control to the rebar
         szItem           = "URL"
         trbbi.cbSize     = SIZEOF(trbbi)
         trbbi.fMask      = %RBBIM_STYLE OR %RBBIM_TEXT OR %RBBIM_CHILD OR _
                            %RBBIM_CHILDSIZE OR %RBBIM_SIZE OR %RBBIM_ID OR _
                            %RBBIM_IDEALSIZE
         trbbi.fStyle     = %RBBS_FIXEDSIZE OR %RBBS_CHILDEDGE
         trbbi.lpText     = VARPTR(szItem)
         trbbi.hWndChild  = hCtrl
         trbbi.cxMinChild = 350
         trbbi.cyMinChild = 21
         trbbi.cx         = 350
         trbbi.wID        = %IDS_STRING1
         trbbi.cxIdeal    = 350
         SendMessage hWndRebar, %RB_INSERTBAND, -1, BYVAL VARPTR(trbbi)

         ' Create the GoBtn text button
         hCtrl = CreateWindowEx(%NULL, _                                             ' extended styles
                                    "Button", _                                          ' class name
                                    "Go", _                                              ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _         ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER OR _     ' class styles
                                    %BS_FLAT, _
                                    493, 2, _                                            ' left, top
                                    34, 24, _                                            ' width, height
                                    hWnd, %IDC_GOBTN, _                            ' handle of parent, control ID
                                    GetModuleHandle(""), BYVAL %NULL)                    ' handle of instance, creation parameters
         SendMessage hCtrl, %WM_SETFONT, hFont, %TRUE

         ' Add the band containing the GoBtn text button to the rebar
         trbbi.cbSize     = SIZEOF(trbbi)
         trbbi.fMask      = %RBBIM_STYLE OR %RBBIM_CHILD OR %RBBIM_CHILDSIZE OR _
                            %RBBIM_SIZE OR %RBBIM_ID OR %RBBIM_IDEALSIZE
         trbbi.fStyle     = %RBBS_FIXEDSIZE OR %RBBS_CHILDEDGE
         trbbi.hWndChild  = hCtrl
         trbbi.cxMinChild = 34
         trbbi.cyMinChild = 24
         trbbi.cx         = 34
         trbbi.wID        = %IDS_STRING2
         trbbi.cxIdeal    = 34
         SendMessage hWndRebar, %RB_INSERTBAND, -1, BYVAL VARPTR(trbbi)

         ' Create the tab control
         ghTabMdi = CreateTabMdiCtl(hWnd, hFont)

         ' Create MDI Client window
         cc.idFirstChild = 1
         cc.hWindowMenu = hMenuWindow  ' For file list in Window menu
         ghWndClient = CreateWindowEx(%WS_EX_CLIENTEDGE, "MDICLIENT", BYVAL %NULL, _
                                      %WS_CHILD OR %WS_CLIPCHILDREN OR %WS_VISIBLE OR %WS_VSCROLL OR %WS_HSCROLL, _
                                      0, 0, 0, 0, hWnd, &H0CAC, GetModuleHandle(""), cc)

   END SELECT

   FUNCTION = DefFrameProc(hWnd, ghWndClient, uMsg, wParam, lParam)

END FUNCTION
' ========================================================================================

' ========================================================================================
' MDI window callback
' ========================================================================================
FUNCTION MdiWindowProc (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, _
                        BYVAL wParam AS DWORD, BYVAL lParam AS LONG) AS LONG

   LOCAL hr            AS LONG
   LOCAL nTab          AS LONG
   LOCAL rc            AS RECT
   LOCAL szText        AS ASCIIZ * 255
   LOCAL hWB           AS DWORD
   LOCAL pIWebBrowser2 AS IWebBrowser2
   LOCAL pWBEvents     AS DWebBrowserEvents2Impl
   LOCAL dwCookie      AS DWORD

   SELECT CASE wMsg

      CASE %WM_CREATE
         GetClientRect hWnd, rc
         ' Create an instance of the WebBrowser control
         hWB = CreateWebBrowser(hWnd)
         ' Get the IDispatch of the control
         pIWebBrowser2 = OC_GetDispatch(hWB)
         IF ISOBJECT(pIWebBrowser2) THEN
            ' Connect to the events fired by the control
            pWBEvents = CLASS "CDWebBrowserEvents2"
            IF ISOBJECT(pWBEvents) THEN
               EVENTS FROM pIWebBrowser2 CALL pWBEvents
               ' Store the cookie in the control properties
               pWBEvents.AddRef   ' Increment the reference count
               dwCookie = OBJPTR(pWBEvents)
               SetProp hWB, "COOKIE", dwCookie
               pWBEvents = NOTHING
            END IF
            ' Release the interface
            pIWebBrowser2 = NOTHING
         END IF
         ' Increase the tab number and set the caption
         ' Insert new tab
         nTab = SendMessage(ghTabMdi, %TCM_GETITEMCOUNT, 0, 0)
         szText = FORMAT$(nTab + 1, "000")
         SetWindowText(hWnd, szText)
         InsertTabMdiItem(ghTabMdi, nTab + 1, szText)
         ' Set it as the current selection on the tab control
         SendMessage(ghTabMdi, %TCM_SETCURSEL, nTab, 0)

      CASE %WM_SIZE
         MoveWindow GetDlgItem(hWnd, %IDC_IEWB), 0, 0, LO(WORD, lParam), HI(WORD, lParam), %TRUE

      CASE %WM_SETFOCUS
         SetFocus GetActiveWbWindow

      CASE %WM_MDIACTIVATE
         ' Using handle, get associated tab via caption string
         ' Get the caption text of the window
         GetWindowText(lParam, szText, %MAX_PATH)
         ' Find the associated tab and activate it
         EnumMdiTitleToTab(szText)
         SetFocus GetActiveWbWindow

      CASE %WM_DESTROY
         ' Get the handle of the window that hosts the control
         hWB = GetDlgItem(hWnd, %IDC_IEWB)
         ' Disconnect events and remove property
         IF hWB THEN
            ' Get the IDispatch of the control
            pIWebBrowser2 = OC_GetDispatch(hWB)
            IF ISOBJECT(pIWebBrowser2) THEN
               ' Retrieve the events cookie
               dwCookie = GetProp(hWB, "COOKIE")
               IF dwCookie THEN
                  POKE DWORD, VARPTR(pWBEvents), dwCookie
                  ' Disconnect events
                  IF ISOBJECT(pWBEvents) THEN
                     EVENTS END pWBEvents
                     pWBEvents = NOTHING
                  END IF
               END IF
               ' Remove the cookie from the control
               RemoveProp hWB, "COOKIE"
            END IF
         END IF
            ' Remove Tab associated with this window
         GetWindowText(hWnd, szText, SIZEOF(szText))
         nTab = EnumMdiTitleToTabRemove(szText)

   END SELECT

   FUNCTION = DefMDIChildProc(hWnd, wMsg, wParam, lParam)

END FUNCTION
' ========================================================================================


' ########################################################################################
' Class CDWebBrowserEvents2
' Interface name = DWebBrowserEvents2
' IID = {34A715A0-6587-11D0-924A-0020AFC7AC4D}
' Web Browser Control events interface
' Attributes = 4112 [&H1010] [Hidden] [Dispatchable]
' ########################################################################################

CLASS CDWebBrowserEvents2 GUID$("{700B73A2-CCFC-4FE0-B9AC-D5853D71B7B9}") AS EVENT

   ' // Indicates that the qualified url has already been
   ' // set in the URL edit textbox
   INSTANCE bQualifiedUrl AS LONG
   ' // Indicates that the NewWindow3 event as been processed
   INSTANCE bNewWindow3 AS LONG

INTERFACE DWebBrowserEvents2Impl GUID$("{34A715A0-6587-11D0-924A-0020AFC7AC4D}") AS EVENT

   INHERIT IDispatch

   ' =====================================================================================
   METHOD StatusTextChange <102> ( _
     BYVAL strText AS STRING _                          ' __in BSTR Text
   )                                                    ' void

      ' Retrieve the handle of the main window
      LOCAL hWnd AS DWORD
      hWnd = phnxGetFormHandle(GetFocus())
      ' Retrieve the handle of the status bar
      LOCAL hStatusBar AS DWORD
      hStatusBar = GetDlgItem(GetParent(ghWndClient), %IDC_STATUSBAR)
      IF ISFALSE hStatusBar THEN EXIT METHOD
      ' Show the changed statusbar text
      strText = ACODE$(strText)
      SendMessage (hStatusBar, %SB_SETTEXT, 0, STRPTR(strText))

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD ProgressChange <108> ( _
     BYVAL Progress AS LONG _                           ' __in long Progress
   , BYVAL ProgressMax AS LONG _                        ' __in long ProgressMax
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD CommandStateChange <105> ( _
     BYVAL nCommand AS LONG _                           ' __in long Command
   , BYVAL bEnable AS INTEGER _                         ' __in VARIANT_BOOL Enable
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD DownloadBegin <106>

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD DownloadComplete <104>

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD TitleChange <113> ( _
     BYVAL strText AS STRING _                          ' __in BSTR Text
   )                                                    ' void

      ' Change the caption of the window and the tab
      LOCAL nTab AS LONG
      LOCAL szText AS ASCIIZ * 255
      nTab = SendMessage(ghTabMdi, %TCM_GETCURSEL, 0, 0)
      szText = FORMAT$(nTab + 1, "000 - ") & ACODE$(strText)
      IF LEN(szText) > %MAX_TABCAPTION_LEN THEN szText = LEFT$(szText, %MAX_TABCAPTION_LEN - 3) & "..."
      SetTabName nTab, szText
      SetWindowText MdiGetActive(ghWndClient), szText

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD PropertyChange <112> ( _
     BYVAL strProperty AS STRING _                      ' __in BSTR szProperty
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD BeforeNavigate2 <250> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   , BYREF vURL AS VARIANT _                            ' __in VARIANT* URL
   , BYREF vFlags AS VARIANT _                          ' __in VARIANT* Flags
   , BYREF vTargetFrameName AS VARIANT _                ' __in VARIANT* TargetFrameName
   , BYREF vPostData AS VARIANT _                       ' __in VARIANT* PostData
   , BYREF vHeaders AS VARIANT _                        ' __in VARIANT* Headers
   , BYREF bCancel AS INTEGER _                         ' __in_out VARIANT_BOOL* Cancel
   )                                                    ' void

   IF bQualifiedUrl = %FALSE THEN
      ' Retrieve the handle of the main window
      LOCAL hwnd AS DWORD
      hwnd = GetParent(ghWndClient)
      ' Retrieve the handle of the rebar control
      LOCAL hCtrl AS DWORD
      hCtrl = GetDlgItem(hwnd, %IDC_REBAR)
      ' Retrieve the handle of the edit control
      hCtrl = GetDlgItem(hCtrl, %IDC_EDITURL)
      ' Show the canonicalized and qualified URL
      SetWindowText hCtrl, VARIANT$(vURL)
      bQualifiedUrl = %TRUE
   END IF

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD NewWindow2 <251> ( _
     BYREF ppDisp AS IDispatch _                        ' __in_out IDispatch** ppDisp
   , BYREF bCancel AS INTEGER _                         ' __in-out VARIANT_BOOL* Cancel
   )                                                    ' void

      ' Cancel new window creation if the NewWindow3 event has been processed.
      IF bNewWindow3 THEN
         bCancel = %VARIANT_TRUE
         bNewWindow3 = %FALSE
         EXIT METHOD
      END IF

      '  Create the new window in a new MDI child window
      LOCAL hMdi AS DWORD
      LOCAL hWB AS DWORD
      LOCAL pIWebBrowser2 AS IWebBrowser2

      ' Create an MDI child
      hMdi = CreateMdiChild("WB", ghWndClient, "", %WS_MAXIMIZE)
      ' Get the handle of the control
      hWB = GetDlgItem(hMdi, %IDC_IEWB)
      ' Get the IDispatch of the control
      pIWebBrowser2 = OC_GetDispatch(hWB)
      IF ISOBJECT(pIWebBrowser2) THEN
         ' Register the browser
         pIWebBrowser2.RegisterAsBrowser = %VARIANT_TRUE
         ppDisp = pIWebBrowser2
         pIWebBrowser2 = NOTHING
      END IF

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD NavigateComplete2 <252> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   , BYREF vURL AS VARIANT _                            ' __in VARIANT* URL
   )                                                    ' void

      ' Retrieve the handle of the main window
      LOCAL hwnd AS DWORD
      hwnd = GetParent(ghWndClient)
      ' Retrieve the handle of the status bar
      LOCAL hStatusBar AS DWORD
      hStatusBar = GetDlgItem(hwnd, %IDC_STATUSBAR)
      IF ISFALSE hStatusBar THEN EXIT METHOD
      ' Retrieve the handle of the rebar control
      LOCAL hCtrl AS DWORD
      hCtrl = GetDlgItem(hwnd, %IDC_REBAR)
      ' Show the changed statusbar text
      LOCAL strURL AS STRING
      strURL = VARIANT$(vURL)
      SendMessage (hStatusBar, %SB_SETTEXT, 0, BYVAL STRPTR(strURL))

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD DocumentComplete <259> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   , BYREF vURL AS VARIANT _                            ' __in VARIANT* URL
   )                                                    ' void

      bQualifiedUrl = %FALSE

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnQuit <253>

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnVisible <254> ( _
     BYVAL bVisible AS INTEGER _                        ' __in VARIANT_BOOL Visible
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnToolBar <255> ( _
     BYVAL bToolBar AS INTEGER _                        ' __in VARIANT_BOOL ToolBar
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnMenuBar <256> ( _
     BYVAL bMenuBar AS INTEGER _                        ' __in VARIANT_BOOL MenuBar
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnStatusBar <257> ( _
     BYVAL bStatusBar AS INTEGER _                      ' __in VARIANT_BOOL StatusBar
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnFullScreen <258> ( _
     BYVAL bFullScreen AS INTEGER _                     ' __in VARIANT_BOOL FullScreen
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD OnTheaterMode <260> ( _
     BYVAL bTheaterMode AS INTEGER _                    ' __in VARIANT_BOOL TheaterMode
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowSetResizable <262> ( _
     BYVAL bResizable AS INTEGER _                      ' __in VARIANT_BOOL Resizable
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowSetLeft <264> ( _
     BYVAL nLeft AS LONG _                              ' __in long Left
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowSetTop <265> ( _
     BYVAL Top AS LONG _                                ' __in long Top
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowSetWidth <266> ( _
     BYVAL nWidth AS LONG _                             ' __in long Width
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowSetHeight <267> ( _
     BYVAL Height AS LONG _                             ' __in long Height
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowClosing <263> ( _
     BYVAL bIsChildWindow AS INTEGER _                  ' __in     VARIANT_BOOL IsChildWindow
   , BYREF bCancel AS INTEGER _                         ' __in_out VARIANT_BOOL* Cancel
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD ClientToHostWindow <268> ( _
     BYREF CX AS LONG _                                 ' __in_out long* CX
   , BYREF CY AS LONG _                                 ' __in_out long* CY
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD SetSecureLockIcon <269> ( _
     BYVAL SecureLockIcon AS LONG _                     ' _in long SecureLockIcon
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD FileDownload <270> ( _
     BYVAL bActiveDocument AS INTEGER _                 ' __in     VARIANT_BOOL ActiveDocument
   , BYREF bCancel AS INTEGER _                         ' __in_out VARIANT_BOOL* Cancel
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD NavigateError <271> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   , BYREF vURL AS VARIANT _                            ' __in VARIANT* URL
   , BYREF vFrame AS VARIANT _                          ' __in VARIANT* Frame
   , BYREF vStatusCode AS VARIANT _                     ' __in VARIANT* StatusCode
   , BYREF bCancel AS INTEGER _                         ' __in_out VARIANT_BOOL* Cancel
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD PrintTemplateInstantiation <225> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD PrintTemplateTeardown <226> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD UpdatePageStatus <227> ( _
     BYVAL pDisp AS IDispatch _                         ' __in IDispatch* pDisp
   , BYREF nPage AS VARIANT _                           ' __in VARIANT* nPage
   , BYREF fDone AS VARIANT _                           ' __in VARIANT* fDone
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD PrivacyImpactedStateChange <272> ( _
     BYVAL bImpacted AS INTEGER _                       ' __in VARIANT_BOOL bImpacted
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD NewWindow3 <273> ( _
     BYREF ppDisp AS IDispatch _                        ' __in_out IDispatch** ppDisp
   , BYREF bCancel AS INTEGER _                         ' __in_out VARIANT_BOOL* Cancel
   , BYVAL dwFlags AS DWORD _                           ' __in unsigned long dwFlags
   , BYVAL bstrUrlContext AS STRING _                   ' __in BSTR bstrUrlContext
   , BYVAL bstrUrl AS STRING _                          ' __in BSTR bstrUrl
   )                                                    ' void

      LOCAL hr AS LONG
      LOCAL hMdi AS DWORD
      LOCAL hWB AS DWORD
      LOCAL pIWebBrowser2 AS IWebBrowser2
      LOCAL strUrl AS STRING
      LOCAL strUrlContext AS STRING

      ' The combination of the %NWMF_FIRST_USERINITED and %NWMF_USERINITED flags
      ' indicates that is the first query that results of a user-initiated action
      ' (a mouse click or key press). Otherwise, it must be a popup.
      IF (dwFlags AND %NWMF_FIRST_USERINITED) = %NWMF_FIRST_USERINITED AND _
         (dwFlags AND %NWMF_USERINITED) = %NWMF_USERINITED THEN
         ' Flag that indicates that this event has been processed.
         bNewWindow3 = %TRUE
         ' Create an MDI child
         hMdi = CreateMdiChild("WB", ghWndClient, "", %WS_MAXIMIZE)
         ' Get the handle of the control
         hWB = GetDlgItem(hMdi, %IDC_IEWB)
         ' Get the IDispatch of the control
         pIWebBrowser2 = OC_GetDispatch(hWB)
         IF ISOBJECT(pIWebBrowser2) THEN
            ' Register the browser
            pIWebBrowser2.RegisterAsBrowser = %VARIANT_TRUE
            ppDisp = pIWebBrowser2
            pIWebBrowser2 = NOTHING
         END IF
      ELSE
         ' Popup blocker.
         strUrl = ACODE$(bstrUrl)
         ' Bug: IE6 gives "?ttp://" instead of "http://"
         IF LEFT$(strUrl, 1) <> "h" THEN MID$(strUrl, 1, 1) = "h"
         hr = MessageBox(GetParent(ghWndClient), "Do you want to activate the popup window?" & $CRLF & strUrl, _
              "Popup blocked", %MB_YESNOCANCEL OR %MB_ICONQUESTION OR %MB_APPLMODAL)
         IF hr = %IDYES THEN
            ' Allow the creation of a new window in the NewWindow2 event
            bNewWindow3 = %FALSE
            bCancel = %VARIANT_FALSE
         ELSE
            bCancel = %VARIANT_TRUE
         END IF
      END IF

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD SetPhishingFilterStatus <282> ( _
     BYVAL PhishingFilterStatus AS LONG _               ' __in long PhishingFilterStatus
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

   ' =====================================================================================
   METHOD WindowStateChanged <283> ( _
     BYVAL dwWindowStateFlags AS DWORD _                ' __in unsigned long dwWindowStateFlags
   , BYVAL dwValidFlagsMask AS DWORD _                  ' __in unsigned long dwValidFlagsMask
   )                                                    ' void

     ' *** Insert your code here ***

   END METHOD
   ' =====================================================================================

END INTERFACE

END CLASS
' ========================================================================================
